@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 2.0.1
+;; Version: 2.1.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 (require 'cl-lib)
 (require 'phpinspect-fs)
 (require 'phpinspect-util)
+(require 'phpinspect-type)
 (require 'phpinspect-pipeline)
 (require 'json)
 
@@ -93,6 +94,19 @@
 qualified names congruent with a bareword type name. Keyed by
 bareword typenames."))
 
+;; FIXME: This is another scenario where an LRU Cache might come in handy (we
+;; don't want to re-compare string prefixes everytime the same namespace is
+;; checked).
+(defun phpinspect-autoloader-get-own-types-in-namespace (al namespace)
+  (cl-assert (stringp namespace))
+  (let ((namespace-fqn (phpinspect--resolve-type-name nil nil namespace))
+	types)
+    (dolist (name (hash-table-keys (phpinspect-autoloader-own-types al)))
+      (when (string-prefix-p namespace-fqn (phpinspect-name-string name))
+	(push (phpinspect--make-type-generated :name-symbol name :fully-qualified t)
+	      types)))
+    types))
+
 (cl-defmethod phpinspect--read-json-file (fs file)
   (with-temp-buffer
     (phpinspect-fs-insert-file-contents fs file)
@@ -188,6 +202,9 @@ the vendor directory."
       (push type-fqn bag)
       (puthash type-name bag (phpinspect-autoloader-type-name-fqn-bags al)))))
 
+(defun phpinspect-autoloader-get-type-names (al)
+  (hash-table-keys (phpinspect-autoloader-type-name-fqn-bags al)))
+
 (cl-defmethod phpinspect-autoloader-get-type-bag ((al phpinspect-autoloader) (type-name (head phpinspect-name)))
   (gethash type-name (phpinspect-autoloader-type-name-fqn-bags al)))
 
