@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 2.0.1
+;; Version: 2.1.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -299,6 +299,10 @@ linked with."
           (dolist (deleted deleted-classes)
             (if (and (setq class (phpinspect-buffer-get-index-for-token
                                   buffer (phpinspect-meta-token deleted)))
+                     ;; If we can find a declaration in the buffer that is equal
+                     ;; to the deleted declaration, the class has just been
+                     ;; edited in a way were the declaration ended up getting
+                     ;; reparsed. No need to delete it from the index
                      (setq replaced (assoc (phpi-typedef-declaration class) new-declarations #'equal)))
                 (pcase-let ((`(,imports ,namespace-name) (phpinspect-get-token-index-context namespaces buffer-imports (cdr replaced))))
                   (phpinspect-buffer-update-index-reference-for-token
@@ -580,7 +584,9 @@ continuing execution."
   (phpinspect-edtrack-register-edit
    (phpinspect-buffer-edit-tracker buffer) start end pre-change-length))
 
-(cl-defmethod phpinspect-buffer-tokens-enclosing-point ((buffer phpinspect-buffer) point)
+(defun phpinspect-buffer-tokens-enclosing-point (buffer point)
+  "Return token metadata objects for tokens enclosing POINT in BUFFER."
+  (cl-assert (phpinspect-buffer-p buffer))
   (phpinspect-bmap-tokens-overlapping (phpinspect-buffer-map buffer) point))
 
 (cl-defmethod phpinspect-buffer-token-meta ((buffer phpinspect-buffer) token)
@@ -629,4 +635,26 @@ use."
      (with-current-buffer (phpinspect-buffer-buffer ,buffer)
        ,body))))
 
+(defun phpinspect-buffer-get-resolvecontext (buffer point)
+  (phpinspect-get-resolvecontext
+   (phpinspect-buffer-project buffer) (phpinspect-buffer-parse-map buffer) point))
+
+
+(defun phpinspect-claim-buffer (buffer &optional project)
+  "Setup an instance of `phpinspect-buffer' for BUFFER.
+
+Like `phpinspect-make-buffer', but arguments are not a
+plist. Sets up buffer-local variable `phpinspect-current-buffer'
+and adds `phpinspect-after-change-function' to buffer-local
+`after-change-functions'.
+
+BUFFER must be a normal emacs buffer.
+If provided, PROJECT must be an instance of `phpinspect-project'."
+  (with-current-buffer buffer
+    (setq-local phpinspect-current-buffer
+		(phpinspect-make-buffer :buffer buffer :-project project))
+    (add-hook 'after-change-functions #'phpinspect-after-change-function nil t)
+
+    phpinspect-current-buffer))
+
 (provide 'phpinspect-buffer)
