@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 2.0.1
+;; Version: 2.1.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -89,7 +89,7 @@ candidate. Candidates can be indexed functions and variables.")
 
 (cl-defmethod phpinspect--completion-list-strings
   ((comp-list phpinspect--completion-list))
-  (let ((strings))
+  (let (strings)
     (obarray-map (lambda (sym) (push (symbol-name sym) strings))
                  (phpinspect--completion-list-completions comp-list))
     strings))
@@ -191,19 +191,75 @@ belonging to a token that conforms with `phpinspect-attrib-p'"
 (cl-defstruct (phpinspect-comp-word (:constructor phpinspect-make-comp-word))
   "Comletion strategy for bare words")
 
+(defun phpinspect-keyword-body-but-not-function-p (token)
+  (and (phpinspect-keyword-body-p token)
+       (not (phpinspect-function-p token))))
+
 (cl-defmethod phpinspect-comp-strategy-supports
   ((_strat phpinspect-comp-word) (q phpinspect-completion-query)
-   (_rctx phpinspect--resolvecontext))
-  (when-let ((subject (phpinspect-completion-subject-at-point
-                       (phpinspect-completion-query-buffer q)
-                       (phpinspect-completion-query-point q)
-                       #'phpinspect-word-p)))
-    (list (phpinspect-meta-start subject) (phpinspect-meta-end subject))))
+   (rctx phpinspect--resolvecontext))
+  "Determine suitability of comp-word strategy to resolve Q.
+
+Words can be type names, function names and keywords.  This
+strategy is a bit special in the sense that it needs to carefully
+consider whether or not it is useful to complete words at point.
+Completing words in a comment for example, is usually not useful."
+  ;; Don't complete words when editing comments
+  (unless (seq-find (phpinspect-meta-token-predicate #'phpinspect-comment-p)
+                    (phpinspect-buffer-tokens-enclosing-point
+                     (phpinspect-completion-query-buffer q)
+                     (phpinspect-completion-query-point q)))
+    (or
+     ;; Complete word being typed
+     (when-let ((subject (phpinspect-completion-subject-at-point
+                          (phpinspect-completion-query-buffer q)
+                          (phpinspect-completion-query-point q)
+                          #'phpinspect-word-p)))
+       (list (phpinspect-meta-start subject) (phpinspect-meta-end subject)))
+
+     ;; Point is right after a "new" token. Complete "newable" types.
+     (when-let ((token-before (phpinspect-bmap-last-token-before-point
+                               (phpinspect-buffer-map
+                                (phpinspect-completion-query-buffer q))
+                               (phpinspect-completion-query-point q)))
+                ((phpinspect-new-p (phpinspect-meta-token token-before))))
+       (list (phpinspect-completion-query-point q)
+             (phpinspect-completion-query-point q)))
+
+   ;; Point is after a scope, static, declaration, const or other keyword that
+   ;; can precede a type or another word.
+   (and (or
+         (phpinspect-keyword-body-but-not-function-p
+          (car (phpinspect--resolvecontext-enclosing-tokens rctx)))
+         (phpinspect-keyword-body-but-not-function-p
+          (car (phpinspect--resolvecontext-subject rctx))))
+
+        (list (phpinspect-completion-query-point q)
+              (phpinspect-completion-query-point q))))))
 
 (cl-defmethod phpinspect-comp-strategy-execute
-  ((_strat phpinspect-comp-word) (_q phpinspect-completion-query)
+  ((_strat phpinspect-comp-word) (q phpinspect-completion-query)
    (rctx phpinspect--resolvecontext))
-  (phpinspect-suggest-functions rctx))
+  (if-let ((subject (phpinspect-bmap-last-token-before-point
+                     (phpinspect-buffer-map
+                      (phpinspect-completion-query-buffer q))
+                     (phpinspect-completion-query-point q)))
+           (parent (car (phpinspect--resolvecontext-enclosing-metadata rctx)))
+           ;; Parent is a declaration token
+           ((phpinspect-declaration-p (phpinspect-meta-token parent)))
+           (grandparent (phpinspect-meta-parent parent))
+           ;; Grandparent is a function, so we're in a function declaration
+           ((phpinspect-function-p (phpinspect-meta-token grandparent)))
+           ;; We're in a function declaration, before the argument list. We're
+           ;; probably editing a function name, so it is not helpful to complete
+           ;; here.
+           ((seq-find #'phpinspect-list-p
+                      (mapcar #'phpinspect-meta-token
+                              (phpinspect-meta-right-siblings subject)))))
+      ;; Return nil, as completing words to name unnamed functions is not
+      ;; useful.
+      nil
+    (phpinspect-suggest-words-at-point rctx)))
 
 (defvar phpinspect-completion-strategies (list (phpinspect-make-comp-attribute)
                                                (phpinspect-make-comp-sigil)
@@ -381,6 +437,13 @@ Returns list of `phpinspect--completion'."
 (cl-defmethod phpinspect--make-completion ((completion-candidate phpinspect-property))
   (phpinspect--make-completion (phpi-prop-definition completion-candidate)))
 
+(cl-defmethod phpinspect--make-completion ((type phpinspect--type))
+  (phpinspect--construct-completion
+   :value (propertize (phpinspect--type-bare-name type) 'face 'font-lock-type-face)
+   :meta (phpinspect--format-type-name type)
+   :target type
+   :kind 'class))
+
 (cl-defmethod phpinspect--make-completion
   ((completion-candidate phpinspect--variable))
   (phpinspect--construct-completion
@@ -395,6 +458,13 @@ Returns list of `phpinspect--completion'."
                             phpinspect--null-type)))
    :kind 'variable))
 
+(cl-defmethod phpinspect--make-completion ((keyword phpinspect-suggest-keyword))
+  (phpinspect--construct-completion
+   :value (propertize (phpinspect-suggest-keyword-word keyword) 'face 'font-lock-keyword-face)
+   :target keyword
+   :kind 'keyword
+   :meta "keyword"))
+
 (define-inline phpinspect--prefix-for-completion (completion)
   (inline-letevals (completion)
     (inline-quote
