@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 2.0.1
+;; Version: 2.1.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -398,4 +398,39 @@ that there are import (\"use\") statements for them."
               (phpinspect-format-use-statements buffer (phpinspect-find-first-use parent))
               (phpinspect-buffer-parse buffer 'no-interrupt)))))))
 
+(defun phpinspect-project-read-type-name (prompt &optional project)
+  "Read a non-fully-qualified type-name using `completing-read'.
+
+PROMPT is passed to `completing-read'.
+
+PROJECT, if provided should be an instance of
+`phpinspect-project'.  If PROJECT is nil, the active project is
+determined using `phpinspect-current-project'."
+  (unless project (setq project (phpinspect-current-project)))
+
+  (phpinspect-intern-name
+   (completing-read
+    prompt
+    (phpinspect-names-to-alist
+     (phpinspect-autoloader-get-type-names
+      (phpinspect-project-autoload project)))
+    nil 'require-match nil 'phpinspect--project-type-name-history)))
+
+(defun phpinspect-insert-type (type-name)
+  "Insert TYPE-NAME as string into current buffer.
+
+This function adds a use statement for TYPE-NAME when none is found."
+  (interactive (list (phpinspect-project-read-type-name "Select a Type: ")))
+  (if phpinspect-current-buffer
+      (let* ((rctx (phpinspect-buffer-get-resolvecontext
+                    phpinspect-current-buffer (point)))
+             (type-resolver (phpinspect--make-type-resolver-for-resolvecontext rctx)))
+        (unless (phpinspect-type-resolver-get-import type-resolver type-name)
+          (phpinspect-add-use-interactive
+           type-name phpinspect-current-buffer
+           (phpinspect-buffer-project phpinspect-current-buffer)
+           (phpinspect-buffer-namespace-at-point phpinspect-current-buffer (point))))
+        (insert (phpinspect-name-string type-name)))
+    (phpinspect-message "Not a phpinspect buffer")))
+
 (provide 'phpinspect-imports)
