@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 2.0.1
+;; Version: 2.1.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -282,6 +282,13 @@
 (cl-defmethod phpinspect-meta-first-child ((meta (head meta)))
   (phpinspect-meta-find-child-after meta (- (phpinspect-meta-start meta) 1)))
 
+(defun phpinspect-meta-token-predicate (token-predicate)
+  "Wrap TOKEN-PREDICATE in a closure that operates on metadata.
+
+The returned closure takes a metadata object as argument and then
+calls TOKEN-PREDICATE on its token
+slot (`phpinspect-meta-token')."
+  (lambda (meta) (funcall token-predicate (phpinspect-meta-token meta))))
 
 (defun phpinspect-meta-string (meta)
   (if meta
