@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 2.0.1
+;; Version: 2.1.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -117,8 +117,8 @@ serious performance hits. Enable at your own risk (:")
     (dolist (func (alist-get 'functions (cdr index)))
       (phpinspect-project-set-function project func))))
 
-(cl-defmethod phpinspect-project-add-index ((_project phpinspect-project) index)
-  (cl-assert (not index))
+(cl-defmethod phpinspect-project-add-index ((_project phpinspect-project) _index)
+  (cl-assert (not _index))
   (phpinspect--log "phpinspect-project-add-index: ignoring added nil index"))
 
 (cl-defmethod phpinspect-project-set-function
@@ -358,6 +358,9 @@ before the search is executed."
     (or (phpinspect-project-get-typedef-or-extra project type)
         (phpinspect-project-get-typedef-create project type))))
 
+(defvar phpinspect--project-type-name-history nil
+  "History list to use for `phpinspect-project-read-type-name'")
+
 ;;; INDEX TASK
 (cl-defstruct (phpinspect-index-task
                (:constructor phpinspect-make-index-task-generated))
