@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 2.0.1
+;; Version: 2.1.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -578,8 +578,8 @@ type. (`phpinspect--type-p')"
       (let ((subject (phpinspect--resolvecontext-subject resolvecontext)))
         (setq type
               (cond ((and assume-derived
-                          (phpinspect--match-sequence subject :f #'phpinspect-word-p))
-                     (funcall type-resolver (phpinspect--make-type :name (cadar subject))))
+                          (phpinspect--match-sequence (last subject) :f #'phpinspect-word-p))
+                     (funcall type-resolver (phpinspect--make-type :name (cadar (last subject)))))
 
                     ((phpinspect-namespace-p enclosing-token)
                      (phpinspect-interpret-expression-type-in-context
@@ -632,11 +632,11 @@ CLASS-TOKEN-META as parse result."
        nil))))
 
 (cl-defmethod phpi-typedef-resolve-property-type
-  ((_typedef phpinspect-typedef) (_project phpinspect-project) property-name &rest _ignored)
+  ((_typedef phpinspect-typedef) (_project phpinspect-project) _property-name &rest _ignored)
   ;; Catch-all for cases where one attempts to resolve a nil property
   ;; name. Saves an if-statement for the caller.
   ;; Can't resolve property type when property name is nil, so we do nothing.
-  (cl-assert (not property-name))
+  (cl-assert (not _property-name))
   nil)
 
 (provide 'phpinspect-resolve)
