@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 2.0.1
+;; Version: 2.1.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -57,7 +57,7 @@
                      (puthash (cadr argument)
                               (phpinspect--make-variable :name (cadr argument))
                               variables))))
-                ((phpinspect-block-p potential-variable)
+                ((phpinspect-block-or-list-p potential-variable)
                  (dolist (nested-token (cdr potential-variable))
                    (push nested-token token-list))))))))
 
@@ -75,6 +75,94 @@
 
       variable-list)))
 
+(cl-defstruct (phpinspect-suggest-keyword
+	       (:constructor phpinspect-make-suggest-keyword-generated))
+  (word nil
+	:type string))
+
+(defun phpinspect-make-suggest-keyword (word)
+  (phpinspect-make-suggest-keyword-generated :word word))
+
+(defconst phpinspect-class-body-keywords (list "const" "private" "public" "protected" "static" "function")
+  "Keywords that can be used within a class")
+
+(defconst phpinspect-scope-body-keywords (list "function" "static")
+  "Keywords that can be used within a statement after a scope keyword.")
+
+(defconst phpinspect-static-body-keywords (list "function")
+  "Keywords that can be used within a statement after a static keyword.")
+
+(defun phpinspect-suggest-class-body-keywords ()
+  (mapcar #'phpinspect-make-suggest-keyword phpinspect-class-body-keywords))
+
+(defun phpinspect-suggest-scope-body-keywords ()
+  (mapcar #'phpinspect-make-suggest-keyword phpinspect-scope-body-keywords))
+
+(defun phpinspect-suggest-static-body-keywords ()
+  (mapcar #'phpinspect-make-suggest-keyword phpinspect-static-body-keywords))
+
+(defun phpinspect-suggest-types-at-point (rctx &optional instantiable)
+  (let* ((resolver (phpinspect--make-type-resolver-for-resolvecontext rctx))
+	 ;; FIXME: this does not include types in the stub cache
+	 (types (mapcar #'cdr (phpinspect-type-resolver--types resolver)))
+	 (namespace (phpinspect-type-resolver--namespace resolver)))
+
+    ;; Include types found within the same namespace
+    (when namespace
+      (setq types
+	    (append types
+		    (phpinspect-autoloader-get-own-types-in-namespace
+		     (phpinspect-project-autoload
+		      (phpinspect--resolvecontext-project rctx))
+		     namespace))))
+
+    ;; Native types are not instantiable
+    (if instantiable
+	types
+      (append types phpinspect-native-types))))
+
+(defun phpinspect-suggest-words-at-point (rctx)
+  (let ((parent (or
+		 ;; When the subject is an empty keyword body, use it as
+		 ;; parent. Context is right after the keyword.
+		 (and-let* ((subject (phpinspect--resolvecontext-subject rctx))
+			       ((phpinspect-keyword-body-p (car subject)))
+			       ((length= (car subject) 1))
+			       ((car subject))))
+		 ;; Otherwise, use first enclosing token as usual.
+		 (car (phpinspect--resolvecontext-enclosing-tokens rctx)))))
+    (or (cond ((phpinspect-class-p parent)
+	       (phpinspect-suggest-class-body-keywords))
+	      ((phpinspect-scope-p parent)
+	       (append
+		(phpinspect-suggest-scope-body-keywords)
+		(phpinspect-suggest-types-at-point rctx)))
+	      ((phpinspect-static-p parent)
+	       (phpinspect-suggest-static-body-keywords))
+
+	      ((or
+		;; Inside argument list in function declaration
+		(and (phpinspect-list-p parent)
+			(phpinspect-declaration-p
+			 (cadr (phpinspect--resolvecontext-enclosing-tokens rctx))))
+		;; Class/function declaration
+		(phpinspect-declaration-p parent))
+	       (phpinspect-suggest-types-at-point rctx))
+
+	      ((and-let* ((subject (phpinspect--resolvecontext-subject rctx) )
+			  ((length= subject 1))
+			  ((phpinspect-new-p (car subject)))
+			  ((phpinspect-suggest-types-at-point rctx 'instantiable))))))
+
+	;; Don't complete functions inside function/class declaration
+	(unless (or (phpinspect-declaration-p parent)
+		    ;; No use completing function names inside comments
+		    (seq-find #'phpinspect-comment-p
+			      (phpinspect--resolvecontext-enclosing-tokens rctx)))
+	  (append
+	   (phpinspect-suggest-types-at-point rctx 'instantiable)
+	  (phpinspect-suggest-functions rctx))))))
+
 (defun phpinspect-get-cached-project-typedef-methods (rctx class-fqn &optional static)
   (phpinspect--log "Getting cached project class methods for %s" class-fqn)
   (let ((class (phpinspect-rctx-get-typedef rctx class-fqn 'no-enqueue)))
