@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 2.0.1
+;; Version: 2.1.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -148,22 +148,24 @@ See https://wiki.php.net/rfc/static_return_type ."
 (defun phpinspect--type= (type1 type2)
   (eq (phpinspect--type-name-symbol type1) (phpinspect--type-name-symbol type2)))
 
-(defun phpinspect--types-uniq (types)
-  "Optimized seq-uniq for types."
+(defun phpinspect--types-uniq (types &optional exclude)
+  "Optimized seq-uniq for TYPES.
+
+If EXCLUDE is non-nil, it is expected to be a `phpinspect--type'
+structure to exclude."
+  (setq exclude (and exclude (phpinspect--type-name-symbol exclude)))
+
   (let* (table
          (filtered (cons nil nil))
          (filtered-rear filtered))
     (dolist (type types)
       (let ((name (phpinspect--type-name-symbol type)))
-        (unless (memq name table)
+        (unless (or (memq name table) (eq exclude name))
           (setq filtered-rear (setcdr filtered-rear (cons type nil)))
-          (push name table))))
+            (push name table))))
 
     (cdr filtered)))
 
-
-
-
 (defun phpinspect--resolve-type-name (types namespace type)
   "Get the FQN for TYPE, using TYPES and NAMESPACE as context.
 
@@ -214,6 +216,15 @@ NAMESPACE may be nil, or a string with a namespace FQN."
             (last-token
              (phpinspect--find-class-token last-token))))))
 
+(oclosure-define phpinspect-type-resolver
+  "Function that resolves `phpinspect--type' instances to be fully qualified."
+  ;; The types known within the context this resolver was created from.
+  (types :mutable nil :type list)
+  (namespace :mutable nil :type string))
+
+(defun phpinspect-type-resolver-get-import (resolver name)
+  (assoc-default name (phpinspect-type-resolver--types resolver) #'eq nil))
+
 (defun phpinspect--make-type-resolver (types &optional token-tree namespace)
   "Little wrapper closure to pass around and resolve types with."
   (let* ((inside-class
@@ -221,7 +232,7 @@ NAMESPACE may be nil, or a string with a namespace FQN."
                               (phpinspect--find-class-token token-tree))))
          (inside-class-name
           (and inside-class (phpinspect--get-class-name-from-token inside-class))))
-    (lambda (type)
+    (oclosure-lambda (phpinspect-type-resolver (types types) (namespace namespace)) (type)
       (phpinspect--type-resolve
        types
        namespace
@@ -269,8 +280,8 @@ NAMESPACE may be nil, or a string with a namespace FQN."
         (concat self "<" (phpinspect--format-type-name (phpinspect--type-format-display-name type)) ">")
       self)))
 
-(cl-defmethod phpinspect--display-format-type-name (type)
-  (cl-assert (not type))
+(cl-defmethod phpinspect--display-format-type-name (_type)
+  (cl-assert (not _type))
 
   (phpinspect--display-format-type-name "unknown-type"))
 
