@@ -4,7 +4,7 @@
 
 ;; Author: Hugo Thunnissen <devel@hugot.nl>
 ;; Keywords: php, languages, tools, convenience
-;; Version: 2.0.1
+;; Version: 2.1.0
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -514,7 +514,7 @@ Return value is a list if structures of the type
         (when (phpi-prop-type prop)
           (push (phpi-prop-type prop) deps)))
 
-      (phpinspect--types-uniq deps))))
+      (phpinspect--types-uniq deps (phpi-typedef-name def)))))
 
 
 (provide 'phpinspect-typedef)
